@@ -1,6 +1,9 @@
 from MAVProxy.modules.lib import grapher
 from MAVProxy.modules.lib import multiproc
 
+import errno
+import socket
+
 graph_count = 1
 
 class Graph_UI(object):
@@ -55,7 +58,7 @@ class Graph_UI(object):
         try:
             while self.xlim_pipe[0].poll():
                 xlim = self.xlim_pipe[0].recv()
-        except (EOFError, BrokenPipeError):
+        except Exception:
             return None
         if xlim != self.xlim:
             return xlim
