@@ -14,11 +14,10 @@ class BatteryModule(mp_module.MPModule):
         self.last_battery_announce = 0
         self.last_battery_announce_time = 0
         self.last_battery_cell_announce_time = 0
-        self.battery_level = -1
-        self.voltage_level = -1
-        self.current_battery = -1
-        self.battery2_voltage = -1
-        self.per_cell = 0
+        self.battery_level = {}
+        self.voltage_level = {}
+        self.current_battery = {}
+        self.per_cell = {}
         self.servo_voltage = -1
         self.high_servo_voltage = -1
         self.last_servo_warn_time = 0
@@ -33,71 +32,84 @@ class BatteryModule(mp_module.MPModule):
         self.settings.append(
             MPSetting('vccwarn', float, 4.3, 'Vcc voltage warning level'))
         self.settings.append(MPSetting('numcells', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells2', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells3', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells4', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells5', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells6', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells7', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells8', int, 0, range=(0,50), increment=1))
+        self.settings.append(MPSetting('numcells9', int, 0, range=(0,50), increment=1))
         self.battery_period = mavutil.periodic_event(5)
 
+    def idstr(self, id):
+        return "" if id == 0 else "%u" % (id+1)
+
+    def numcells(self, id):
+        ncells = "numcells" + self.idstr(id)
+        return self.settings.get_setting(ncells).value
+
     def cmd_bat(self, args):
         '''show battery levels'''
-        print("Flight battery:   %u%%" % self.battery_level)
-        if self.settings.numcells != 0:
-            print("%.2f V/cell for %u cells - approx %u%%" % (self.per_cell,
-                                                              self.settings.numcells,
-                                                              self.vcell_to_battery_percent(self.per_cell)))
+        print("Flight battery:   %u%%" % self.battery_level[0])
+        for id in range(9):
+            if self.numcells(id) != 0 and id in self.voltage_level:
+                print("Bat%u %.2f V/cell for %u cells %.1fA %.2f%%" % (id+1,
+                                                                        self.per_cell[id],
+                                                                        self.numcells(id),
+                                                                        self.current_battery[id],
+                                                                        self.battery_level[id]))
 
     def battery_report(self):
-        batt_mon = int(self.get_mav_param('BATT_MONITOR',0))
-
-        #report voltage level only
         battery_string = ''
-        if batt_mon == 3:
-            battery_string = 'Batt: %.2fV' % (float(self.voltage_level) / 1000.0)
-        elif batt_mon >= 4:
-            battery_string = 'Batt: %u%%/%.2fV %.1fA' % (self.battery_level, (float(self.voltage_level) / 1000.0), self.current_battery / 100.0 )
-        if self.battery2_voltage != -1:
-            battery_string += ' %.2fV' % self.battery2_voltage
+        for id in range(9):
+            if not id in self.voltage_level:
+                continue
+            batt_mon = int(self.get_mav_param('BATT%s_MONITOR' % self.idstr(id),0))
+            if batt_mon == 0:
+                continue
 
+            #report voltage level only
+            if batt_mon == 3:
+                battery_string += 'Batt%u: %.2fV ' % (id+1, self.voltage_level[id])
+            elif batt_mon >= 4:
+                battery_string += 'Batt%u: %u%%/%.2fV %.1fA ' % (id+1,
+                                                                   self.battery_level[id],
+                                                                   self.voltage_level[id],
+                                                                   self.current_battery[id])
         self.console.set_status('Battery', battery_string, row=1)
 
-        rbattery_level = int((self.battery_level+5)/10)*10
+        # only announce first battery
+        if not 0 in self.battery_level:
+            return
+
+        batt_mon = int(self.get_mav_param('BATT_MONITOR',0))
+        if batt_mon == 0:
+            return
+
+        rbattery_level = int((self.battery_level[0]+5)/10)*10
         if batt_mon >= 4 and self.settings.battwarn > 0 and time.time() > self.last_battery_announce_time + 60*self.settings.battwarn:
-            self.last_battery_announce_time = time.time()
             if rbattery_level != self.last_battery_announce:
                 self.say("Flight battery %u percent" % rbattery_level, priority='notification')
-                self.last_battery_announce = rbattery_level
-            #check voltage level to ensure we've actually received data about
-            #the battery (prevents false positive warning at startup)
-            if self.voltage_level != -1 and rbattery_level <= 20:
+            if rbattery_level <= 20:
                 self.say("Flight battery warning")
+            self.last_battery_announce_time = time.time()
 
-        if (self.settings.numcells != 0 and self.per_cell < self.settings.batwarncell and
+        if (self.numcells(0) != 0 and 0 in self.per_cell and self.per_cell[0] < self.settings.batwarncell and
             self.settings.battwarn > 0 and time.time() > self.last_battery_cell_announce_time + 60*self.settings.battwarn):
             self.say("Cell warning")
             self.last_battery_cell_announce_time = time.time()
 
 
-    def vcell_to_battery_percent(self, vcell):
-        '''convert a cell voltage to an approximate
-        percentage battery level for a LiPO'''
-        if vcell > 4.1:
-            # above 4.1 is 100% battery
-            return 100.0
-        elif vcell > 3.81:
-            # 3.81 is 17% remaining, from flight logs
-            return 17.0 + 83.0 * (vcell - 3.81) / (4.1 - 3.81)
-        elif vcell > 3.2:
-            # below 3.2 it degrades fast. It's dead at 3.2
-            return 0.0 + 17.0 * (vcell - 3.20) / (3.81 - 3.20)
-        # it's dead or disconnected
-        return 0.0
-
-
-    def battery_update(self, SYS_STATUS):
+    def battery_update(self, BATTERY_STATUS):
         '''update battery level'''
         # main flight battery
-        self.battery_level = SYS_STATUS.battery_remaining
-        self.voltage_level = SYS_STATUS.voltage_battery
-        self.current_battery = SYS_STATUS.current_battery
-        if self.settings.numcells != 0:
-            self.per_cell = (self.voltage_level*0.001) / self.settings.numcells
+        id = BATTERY_STATUS.id
+        self.battery_level[id] = BATTERY_STATUS.battery_remaining
+        self.voltage_level[id] = BATTERY_STATUS.voltages[0]*0.001
+        self.current_battery[id] = BATTERY_STATUS.current_battery*0.01
+        if self.numcells(id) > 0:
+            self.per_cell[id] = self.voltage_level[id] / self.numcells(id)
 
     def power_status_update(self, POWER_STATUS):
         '''update POWER_STATUS warnings level'''
@@ -122,7 +134,7 @@ class BatteryModule(mp_module.MPModule):
     def mavlink_packet(self, m):
         '''handle a mavlink packet'''
         mtype = m.get_type()
-        if mtype == "SYS_STATUS":
+        if mtype == "BATTERY_STATUS":
             self.battery_update(m)
         if mtype == "BATTERY2":
             self.battery2_voltage = m.voltage * 0.001
