@@ -31,6 +31,8 @@ class FakeGPSModule(mp_module.MPModule):
                                            MPMenuItem('SetPos (with alt)', 'SetPosAlt', '# fakegps setpos ',
                                                     handler=MPMenuCallTextDialog(title='Altitude (m)', default=self.mpstate.settings.guidedalt))])
                 map.add_menu(menu)
+        self.position = mp_util.mp_position()
+        self.update_mpstate()
 
     def get_location(self):
         '''access to location for other modules'''
@@ -51,6 +53,14 @@ class FakeGPSModule(mp_module.MPModule):
         else:
             print(usage)
 
+    def update_mpstate(self):
+        '''update mpstate position'''
+        self.position.latitude = self.FakeGPS_settings.lat
+        self.position.longitude = self.FakeGPS_settings.lon
+        self.position.altitude = self.FakeGPS_settings.alt
+        self.position.timestamp = time.time()
+        self.mpstate.position = self.position
+
     def cmd_setpos(self, args):
         '''set pos from map'''
         latlon = self.mpstate.click_location
@@ -62,6 +72,8 @@ class FakeGPSModule(mp_module.MPModule):
         self.FakeGPS_settings.lon = lon
         if len(args) > 0:
             self.FakeGPS_settings.alt = float(args[0])
+        self.update_mpstate()
+
 
     def get_gps_time(self, tnow):
         '''return gps_week and gps_week_ms for current time'''
@@ -77,7 +89,7 @@ class FakeGPSModule(mp_module.MPModule):
 
     def idle_task(self):
         '''called on idle'''
-        if self.master is None:
+        if self.master is None or self.FakeGPS_settings.rate <= 0:
             return
         now = time.time()
         if now - self.last_send < 1.0 / self.FakeGPS_settings.rate:
